<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace PagerdutyModule\Command;

use Cake\Console\Arguments;
use Cake\Console\Command;
use Cake\Console\ConsoleIo;
use Cake\Console\ConsoleOptionParser;
use Cake\ORM\TableRegistry;
use PagerdutyModule\Model\Table\PagerdutyQueuesTable;

/**
 * Class PagerdutyNotificationCommand
 * @package PagerdutyModule\Command
 */
class PagerdutyNotificationCommand extends Command {

    /**
     * @param ConsoleOptionParser $parser
     * @return ConsoleOptionParser
     */
    public function buildOptionParser(ConsoleOptionParser $parser): ConsoleOptionParser {
        /** @var ConsoleOptionParser $parser */
        $parser = parent::buildOptionParser($parser);

        $parser->addOptions([
            'type'             => ['short' => 't', 'help' => __d('oitc_console', 'Type of the notification host or service')],
            'notificationtype' => ['help' => __d('oitc_console', 'Notification type of monitoring engine')],
            'hostname'         => ['help' => __d('oitc_console', 'Host uuid you want to send a notification')],
            'hoststate'        => ['help' => __d('oitc_console', 'current host state')],
            'output'           => ['help' => __d('oitc_console', 'output')],
            'servicedesc'      => ['help' => __d('oitc_console', 'Service uuid you want to notify')],
            'servicestate'     => ['help' => __d('oitc_console', 'service state')],
        ]);

        return $parser;
    }

    /**
     * @param Arguments $args
     * @param ConsoleIo $io
     */
    public function execute(Arguments $args, ConsoleIo $io) {

        if ($args->getOption('type') !== '') {
            if ($args->getOption('type') === 'Host') {
                $notification = [
                    'PagerdutyQueue' => [
                        'host_uuid'        => $args->getOption('hostname'),
                        'host_state'       => (int)$args->getOption('hoststate'),
                        'notificationtype' => $args->getOption('notificationtype'),
                        'type'             => OBJECT_HOST,
                        'output'           => $args->getOption('output'),
                    ]
                ];
            }

            if ($args->getOption('type') === 'Service') {
                $notification = [
                    'PagerdutyQueue' => [
                        'host_uuid'        => $args->getOption('hostname'),
                        'host_state'       => (int)$args->getOption('hoststate'),
                        'service_uuid'     => $args->getOption('servicedesc'),
                        'service_state'    => $args->getOption('servicestate'),
                        'notificationtype' => $args->getOption('notificationtype'),
                        'type'             => OBJECT_SERVICE,
                        'output'           => $args->getOption('output'),
                    ]
                ];
            }

            $this->queueNotification($notification);
        }

    }

    /**
     * @param $notification
     */
    public function queueNotification($notification) {
        /** @var PagerdutyQueuesTable $PagerdutyQueuesTable */
        $PagerdutyQueuesTable = TableRegistry::getTableLocator()->get('PagerdutyModule.PagerdutyQueues');

        $PagerdutyQueueEntity = $PagerdutyQueuesTable->newEntity($notification['PagerdutyQueue']);
        $PagerdutyQueuesTable->save($PagerdutyQueueEntity);
    }
}
