<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace PagerdutyModule\Command;

use App\Model\Table\ProxiesTable;
use App\Model\Table\SystemsettingsTable;
use Cake\Console\Arguments;
use Cake\Console\Command;
use Cake\Console\ConsoleIo;
use Cake\ORM\TableRegistry;
use GuzzleHttp\Client;
use itnovum\openITCOCKPIT\Core\Interfaces\CronjobInterface;
use itnovum\openITCOCKPIT\Core\Views\HoststatusIcon;
use itnovum\openITCOCKPIT\Core\Views\ServicestatusIcon;
use itnovum\openITCOCKPIT\Exceptions\HostNotFoundException;
use itnovum\openITCOCKPIT\Exceptions\ServiceNotFoundException;
use PagerdutyModule\Lib\PagerdutyData;
use PagerdutyModule\Model\Table\PagerdutyQueuesTable;
use PagerdutyModule\Model\Table\PagerdutySettingsTable;

/**
 * Class PagerdutySendqueueCommand
 * @package PagerdutyModule\Command
 */
class PagerdutySendqueueCommand extends Command implements CronjobInterface {

    private $pagerdutySettings;
    private $proxySettings;

    /**
     * @param Arguments $args
     * @param ConsoleIo $io
     */
    public function execute(Arguments $args, ConsoleIo $io) {
        $io->setStyle('green', ['text' => 'green']);
        $io->setStyle('red', ['text' => 'red']);
        $io->out('Push notifications to PagerDuty', 0);
        $io->out('<green>   Ok</green>');
        $this->processQueue();
        $io->hr();
    }

    private function processQueue() {
        /** @var ProxiesTable $Proxy */
        $Proxy = TableRegistry::getTableLocator()->get('Proxies');
        /** @var PagerdutySettingsTable $PagerdutySettingsTable */
        $PagerdutySettingsTable = TableRegistry::getTableLocator()->get('PagerdutyModule.PagerdutySettings');
        /** @var PagerdutyQueuesTable $PagerdutyQueuesTable */
        $PagerdutyQueuesTable = TableRegistry::getTableLocator()->get('PagerdutyModule.PagerdutyQueues');
        /** @var SystemsettingsTable $SystemsettingsTable */
        $SystemsettingsTable = TableRegistry::getTableLocator()->get('Systemsettings');
        /** @var PagerdutyData $PagerdutyData */
        $PagerdutyData = new PagerdutyData();

        $this->proxySettings = $Proxy->getSettings();
        $this->pagerdutySettings = $PagerdutySettingsTable->getPagerdutySettings(true);

        $systemAddress = $SystemsettingsTable->find()
            ->where([
                'key' => 'SYSTEM.ADDRESS'
            ])->first()->toArray()['value'];

        $systemName = $SystemsettingsTable->find()
            ->where([
                'key' => 'FRONTEND.SYSTEMNAME'
            ])->first()->toArray()['value'];


        $recordEntities = $PagerdutyQueuesTable->find('all')->all();
        $records = [];
        if (!empty($recordEntities)) {
            $records = $recordEntities->toArray();
        }
        $ids = [];
        foreach ($records as $record) {
            //https://v2.developer.pagerduty.com/docs/events-api-v2
            if ($this->isHostNotification($record['type'])) {
                try {
                    $host = $PagerdutyData->getHost($record['host_uuid']);
                    $HoststatusIcon = new HoststatusIcon($record['host_state']);
                    $event = [
                        /*
                        'payload'      => [
                            'summary'        => sprintf('[%s] %s', $host['name'], $record['output']),
                            //'timestamp'      => date('c'),
                            'source'         => $record['host_uuid'],
                            'severity'       => 'info',
                            'class'          => $HoststatusIcon->getHumanState(),
                            'custom_details' => [
                                'host_uuid' => $record['host_uuid'],
                            ]
                        ],
                        'routing_key'  => $this->pagerdutySettings['integration_key'],
                        'dedup_key'    => $record['host_uuid'],
                        'dedup_key'    => $record['host_uuid'],
                        'alert_key'    => $record['host_uuid'],
                        'event_action' => 'trigger',
                        'client'       => 'openITCOCKPIT',
                        'client_url'   => sprintf(
                            'https://%s/hosts/browser/%s',
                            $systemAddress,
                            $host['id']
                        )
                        */


                        'service_key'  => $this->pagerdutySettings['integration_key'],
                        'event_type'   => $this->getEventType($record['notificationtype']),
                        'description'  => sprintf('[%s] %s', $host['name'], $record['output']),
                        'incident_key' => $record['host_uuid'],
                        'dedup_key'    => $record['host_uuid'],
                        'details'      => [
                            'output'    => $record['output'],
                            'host_name' => $host['name'],
                            'host_uuid' => $record['host_uuid'],
                            'hoststate' => $HoststatusIcon->getHumanState()
                        ],

                        'client'     => $systemName,
                        'client_url' => sprintf(
                            'https://%s/#!/hosts/browser/%s',
                            $systemAddress,
                            $host['id']
                        ),


                    ];
                } catch (HostNotFoundException $e) {
                    error_log(sprintf('Host with uuid %s not found', $record['host_uuid']));
                    continue;
                }
            } else {
                try {
                    $service = $PagerdutyData->getService($record['service_uuid']);
                    $HoststatusIcon = new HoststatusIcon($record['host_state']);
                    $ServicestatusIcon = new ServicestatusIcon($record['service_state']);
                    $event = [
                        'service_key'  => $this->pagerdutySettings['integration_key'],
                        'event_type'   => $this->getEventType($record['notificationtype']),
                        'description'  => sprintf(
                            '[%s/%s] %s',
                            $service['host']['name'],
                            $service['name'],
                            $record['output']
                        ),
                        'incident_key' => $record['service_uuid'],
                        'dedup_key'    => $record['service_uuid'],
                        'details'      => [
                            'output'       => $record['output'],
                            'host_name'    => $service['host']['name'],
                            'service_name' => $service['name'],
                            'host_uuid'    => $record['host_uuid'],
                            'service_uuid' => $record['service_uuid'],
                            'hoststate'    => $HoststatusIcon->getHumanState(),
                            'servicestate' => $ServicestatusIcon->getHumanState()
                        ],
                        'client'       => $systemName,
                        'client_url'   => sprintf(
                            'https://%s/#!/services/browser/%s',
                            $systemAddress,
                            $service['id']
                        ),
                    ];
                } catch (ServiceNotFoundException $e) {
                    error_log(sprintf('Service with uuid %s not found', $record['service_uuid']));
                    continue;
                }
            }

            $ids[] = $record['id'];
            $this->sendRequest($event);
        }

        if (!empty($ids)) {
            $PagerdutyQueuesTable->deleteAll([
                'id IN' => $ids
            ]);
        }
    }

    private function sendRequest($event) {
        $client = new Client();

        $options = [
            'json'    => $event,
            'headers' => [
                'Accept'        => 'application/vnd.pagerduty+json;version=2',
                'Authorization' => sprintf('Token token=%s', $this->pagerdutySettings['api_key']),
            ]
        ];

        if ($this->pagerdutySettings['use_proxy']) {
            $options['proxy'] = [
                'http'  => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port']),
                'https' => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port'])
            ];
        } else {
            $options['proxy'] = [
                'http'  => false,
                'https' => false
            ];
        }

        $response = $client->post($this->pagerdutySettings['api_url'], $options);

        $body = $response->getBody();
        //print_r($body->getContents());
    }

    private function isHostNotification($type) {
        return $type == OBJECT_HOST;
    }

    /**
     * @param string $notificationtype
     * @return string
     */
    private function getEventType($notificationtype) {
        switch ($notificationtype) {
            case 'PROBLEM':
                return 'trigger';

            case 'ACKNOWLEDGEMENT':
                return 'acknowledge';

            case 'RECOVERY':
                return 'resolve';

            default:
                return 'trigger';
        }
    }
}
