<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace PagerdutyModule\Controller;

use App\Model\Table\ProxiesTable;
use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Core\FileDebugger;
use itnovum\openITCOCKPIT\Core\System\Gearman;
use itnovum\openITCOCKPIT\Exceptions\HostNotFoundException;
use itnovum\openITCOCKPIT\Exceptions\ServiceNotFoundException;
use PagerdutyModule\Lib\PagerdutyData;
use PagerdutyModule\Model\Table\PagerdutySettingsTable;

class AcknowledgeController extends AppController {

    private $proxySettings;
    private $pagerdutySettings;

    /**
     * @throws HostNotFoundException
     * @throws ServiceNotFoundException
     */
    public function submit() {
        /** @var ProxiesTable $Proxy */
        $Proxy = TableRegistry::getTableLocator()->get('Proxies');
        /** @var PagerdutySettingsTable $PagerdutySettingsTable */
        $PagerdutySettingsTable = TableRegistry::getTableLocator()->get('PagerdutyModule.PagerdutySettings');
        /** @var Gearman $GearmanClient */
        $GearmanClient = new Gearman();
        /** @var PagerdutyData $PagerdutyData */
        $PagerdutyData = new PagerdutyData();

        $this->proxySettings = $Proxy->getSettings();
        $this->pagerdutySettings = $PagerdutySettingsTable->getPagerdutySettings(true);


        $data = $this->request->getData();
        if (empty($data)) {
            $message = 'Empty post data';
            $this->set('message', $message);
            $this->set('_serialize', ['message']);
            return;
        }

        foreach ($data['messages'] as $message) {
            if ($message['event'] !== 'incident.acknowledge') {
                $this->set('message', __('Wrong incident type'));
                $this->set('_serialize', ['message']);
                return;
            }
            $incident = $message['incident'];
            $incidentKey = $incident['incident_key'];

            foreach ($incident['acknowledgements'] as $acknowledger) {
                if ($acknowledger['acknowledger']['type'] == 'api') {
                    $authorName = 'API';
                    $authorUrl = null;
                } else {
                    $authorName = $acknowledger['acknowledger']['summary'];
                    $authorUrl = $acknowledger['acknowledger']['html_url'];
                    break;
                }
            }

            $ackUrl = $authorName;
            if ($authorUrl) {
                $ackUrl = sprintf('[url=%s]%s[/url]', $authorUrl, $authorName);
            }

            try {
                $host = $PagerdutyData->getHost($incidentKey);
            } catch (HostNotFoundException $e) {
                $host = [];
            }
            if (!empty($host)) {

                $GearmanClient->sendBackground('cmd_external_command', [
                    'command'     => 'ACKNOWLEDGE_HOST_PROBLEM',
                    'parameters'  => [
                        'hostUuid'   => $host['uuid'],
                        'sticky'     => 2, // 2 = sticky with Nagios Core, 1 = Sticky with Naemon Core, 2 works for both :)
                        'notify'     => 0, // do not enable - loop!
                        'persistent' => 1,
                        'author'     => $authorName,
                        'comment'    => sprintf(
                            'Issue %s was acknowledged by %s due to PagerDuty.',
                            sprintf('[url=%s]%s[/url]', $incident['html_url'], $incident['id']),
                            $ackUrl
                        )
                    ],
                    'satelliteId' => null
                ]);

                $this->set('message', __('Command added successfully to queue'));
                $this->set('_serialize', ['message']);
                return;
            }

            try {
                $service = $PagerdutyData->getService($incidentKey);
            } catch (ServiceNotFoundException $e) {
                $service = [];
            }
            if (!empty($service)) {
                $GearmanClient->sendBackground('cmd_external_command', [
                    'command'     => 'ACKNOWLEDGE_SVC_PROBLEM',
                    'parameters'  => [
                        'hostUuid'    => $service['host']['uuid'],
                        'serviceUuid' => $service['uuid'],
                        'sticky'      => 2, // 2 = sticky with Nagios Core, 1 = Sticky with Naemon Core, 2 works for both :)
                        'notify'      => 0, // do not enable - loop!
                        'persistent'  => 1,
                        'author'      => $authorName,
                        'comment'     => sprintf(
                            'Issue %s was acknowledged by %s due to PagerDuty.',
                            sprintf('[url=%s]%s[/url]', $incident['html_url'], $incident['id']),
                            sprintf('[url=%s]%s[/url]', $authorUrl, $authorName)
                        )
                    ],
                    'satelliteId' => null
                ]);

                $this->set('message', __('Command added successfully to queue'));
                $this->set('_serialize', ['message']);
                return;
            }

        }
        $this->set('message', __('Unknown'));
        $this->set('_serialize', ['message']);
    }

}

