<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, version 3 of the License.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// 2.
//    If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//    under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//    License agreement and license key will be shipped with the order
//    confirmation.

namespace PagerdutyModule\Lib;


use App\Model\Table\HostsTable;
use App\Model\Table\ServicesTable;
use Cake\Datasource\EntityInterface;
use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Exceptions\HostNotFoundException;
use itnovum\openITCOCKPIT\Exceptions\ServiceNotFoundException;

/**
 * Class PagerdutyData
 * @package PagerdutyModule\Lib
 */
class PagerdutyData {

    /**
     * @param string $hostUuid
     * @return array|EntityInterface|null
     * @throws HostNotFoundException
     */
    public function getHost($hostUuid) {
        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');

        $host = $HostsTable->find()
            ->where([
                'Hosts.uuid' => $hostUuid
            ])
            ->select([
                'Hosts.id',
                'Hosts.uuid',
                'Hosts.name',
                'Hosts.address',
                'Hosts.satellite_id'
            ])
            ->first();

        if (empty($host)) {
            throw new HostNotFoundException();
        }
        $host = $host->toArray();

        return $host;
    }

    /**
     * @param string $serviceUuid
     * @return array|EntityInterface|null
     * @throws ServiceNotFoundException
     */
    public function getService($serviceUuid) {
        /** @var ServicesTable $ServicesTable */
        $ServicesTable = TableRegistry::getTableLocator()->get('Services');

        $service = $ServicesTable->find()
            ->contain([
                'Hosts',
                'Servicetemplates'
            ])
            ->where([
                'Services.uuid' => $serviceUuid
            ])
            ->select([
                'Services.id',
                'Services.uuid',
                'Services.name',
                'Servicetemplates.id',
                'Servicetemplates.name',
                'Hosts.id',
                'Hosts.uuid',
                'Hosts.name',
                'Hosts.satellite_id'
            ])
            ->first();

        if (empty($service)) {
            throw new ServiceNotFoundException();
        }
        $service = $service->toArray();

        if ($service['name'] === null || $service['name'] === '') {
            $service['name'] = $service['servicetemplate']['name'];
        }

        return $service;
    }
}