<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace PagerdutyModule\Model\Table;

use Cake\Datasource\EntityInterface;
use Cake\ORM\Behavior\TimestampBehavior;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use PagerdutyModule\Model\Entity\PagerdutyQueue;

/**
 * PagerdutyQueues Model
 *
 * @method PagerdutyQueue get($primaryKey, $options = [])
 * @method PagerdutyQueue newEntity($data = null, array $options = [])
 * @method PagerdutyQueue[] newEntities(array $data, array $options = [])
 * @method PagerdutyQueue|false save(EntityInterface $entity, $options = [])
 * @method PagerdutyQueue saveOrFail(EntityInterface $entity, $options = [])
 * @method PagerdutyQueue patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method PagerdutyQueue[] patchEntities($entities, array $data, array $options = [])
 * @method PagerdutyQueue findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin TimestampBehavior
 */
class PagerdutyQueuesTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('pagerduty_queues');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator {
        $validator
            ->nonNegativeInteger('id')
            ->allowEmptyString('id', null, 'create');

        $validator
            ->scalar('host_uuid')
            ->maxLength('host_uuid', 255)
            ->allowEmptyString('host_uuid');

        $validator
            ->integer('host_state')
            ->allowEmptyString('host_state');

        $validator
            ->scalar('service_uuid')
            ->maxLength('service_uuid', 255)
            ->allowEmptyString('service_uuid');

        $validator
            ->integer('service_state')
            ->allowEmptyString('service_state');

        $validator
            ->scalar('notificationtype')
            ->maxLength('notificationtype', 255)
            ->allowEmptyString('notificationtype');

        $validator
            ->nonNegativeInteger('type')
            ->allowEmptyString('type');

        $validator
            ->scalar('output')
            ->maxLength('output', 255)
            ->allowEmptyString('output');

        return $validator;
    }
}
