<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace PagerdutyModule\Model\Table;

use Cake\Datasource\EntityInterface;
use Cake\ORM\Behavior\TimestampBehavior;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use PagerdutyModule\Model\Entity\PagerdutySetting;

/**
 * PagerdutySettings Model
 *
 * @method PagerdutySetting get($primaryKey, $options = [])
 * @method PagerdutySetting newEntity($data = null, array $options = [])
 * @method PagerdutySetting[] newEntities(array $data, array $options = [])
 * @method PagerdutySetting|false save(EntityInterface $entity, $options = [])
 * @method PagerdutySetting saveOrFail(EntityInterface $entity, $options = [])
 * @method PagerdutySetting patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method PagerdutySetting[] patchEntities($entities, array $data, array $options = [])
 * @method PagerdutySetting findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin TimestampBehavior
 */
class PagerdutySettingsTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('pagerduty_settings');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator {
        $validator
            ->nonNegativeInteger('id')
            ->allowEmptyString('id', null, 'create');

        $validator
            ->scalar('integration_key')
            ->maxLength('integration_key', 255)
            ->allowEmptyString('integration_key');

        $validator
            ->scalar('api_key')
            ->maxLength('api_key', 255)
            ->allowEmptyString('api_key');

        $validator
            ->scalar('api_url')
            ->maxLength('api_url', 255)
            ->allowEmptyString('api_url');

        $validator
            ->boolean('two_way')
            ->notEmptyString('two_way');

        $validator
            ->boolean('use_proxy')
            ->notEmptyString('use_proxy');

        return $validator;
    }

    /**
     * @param bool $toArray
     * @return array|EntityInterface|PagerdutySetting|null
     */
    public function getPagerdutySettings($toArray = false) {
        $default = [
            'api_key'         => null,
            'integration_key' => null,
            'api_url'         => 'https://events.pagerduty.com/generic/2010-04-15/create_event.json',
            'two_way'         => false,
            'use_proxy'       => false
        ];

        $result = $this->find()->first();

        if (empty($result)) {
            $result = $this->newEntity($default);
        }
        if ($toArray) {
            return $result->toArray();
        }

        return $result;
    }
}
